// -*- C++ -*-
/*!
 * @file  ev3touch.cpp
 * @brief ev3 use touch sensor
 * @date $Date$
 *
 * $Id$
 */

#include "ev3touch.h"

// Module specification
// <rtc-template block="module_spec">
static const char* ev3touch_spec[] =
  {
    "implementation_id", "ev3touch",
    "type_name",         "ev3touch",
    "description",       "ev3 use touch sensor",
    "version",           "1.0.0",
    "vendor",            "fsk",
    "category",          "ev3",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.front_speed", "0.05",
    "conf.default.back_speed", "0.05",
    "conf.default.back_time", "1",
    "conf.default.turn_speed", "1.57",
    "conf.default.turn_time", "1",

    // Widget
    "conf.__widget__.front_speed", "text",
    "conf.__widget__.back_speed", "text",
    "conf.__widget__.back_time", "text",
    "conf.__widget__.turn_speed", "text",
    "conf.__widget__.turn_time", "text",
    // Constraints

    "conf.__type__.front_speed", "float",
    "conf.__type__.back_speed", "float",
    "conf.__type__.back_time", "float",
    "conf.__type__.turn_speed", "float",
    "conf.__type__.turn_time", "float",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ev3touch::ev3touch(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_touchIn("touch", m_touch),
    m_velOut("vel", m_vel)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
ev3touch::~ev3touch()
{
}



RTC::ReturnCode_t ev3touch::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("touch", m_touchIn);
  
  // Set OutPort buffer
  addOutPort("vel", m_velOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("front_speed", m_front_vel, "0.05");
  bindParameter("back_speed", m_back_vel, "0.05");
  bindParameter("back_time", m_back_time, "1");
  bindParameter("turn_speed", m_turn_vel, "1.57");
  bindParameter("turn_time", m_turn_time, "1");
  // </rtc-template>
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ev3touch::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ev3touch::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ev3touch::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ev3touch::onActivated(RTC::UniqueId ec_id)
{
	std::cout << "onActive" << std::endl;
	toward = 0;
	changetoward = true;
	m_vel.data.vx = m_front_vel;
	m_vel.data.vy = 0;
	m_vel.data.va = 0;
	f_v = m_front_vel;
	b_v = m_back_vel;
	b_t = m_back_time;
	t_v = m_turn_vel;
	t_t = m_turn_time;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ev3touch::onDeactivated(RTC::UniqueId ec_id)
{
	std::cout << "onDeactive" << std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t ev3touch::onExecute(RTC::UniqueId ec_id)
{
	//RtBM[V̒lύXɕ\
	if (f_v != m_front_vel || b_v != m_back_vel || b_t != m_back_time || t_v != m_turn_vel || t_t != m_turn_time){
		std::cout << "front vel " << m_front_vel << " back vel " << m_back_vel << " time " << m_back_time;
		std::cout << "turn vel " << m_turn_vel << " time" << m_turn_time << std::endl;

		m_vel.data.vx = m_front_vel;
		f_v = m_front_vel;
		b_v = m_back_vel;
		b_t = m_back_time;
		t_v = m_turn_vel;
		t_t = m_turn_time;
	}


	//^b`ZT[̒lǂݍ
	if (m_touchIn.isNew())
	{
		m_touchIn.read();
		if (changetoward == true){
			if (m_touch.data[0] == false && m_touch.data[1] == false)toward = 0;
			else if (m_touch.data[0] == true && m_touch.data[1] == false)toward = 1;
			else if (m_touch.data[0] == false && m_touch.data[1] == true)toward = 2;
			else toward = 3;
			if (toward != 0){
				changetoward = false;
				std::cout << "touch:" << toward << std::endl;

			}
		}
	}
	//̏
	if (changetoward == false){

		m_vel.data.vx = -m_back_vel;
		m_vel.data.vy = 0;
		m_vel.data.va = 0;
		m_velOut.write();
		std::cout << " back" << std::endl;
		Sleep(b_t * 1000);

		if (toward == 1){
			m_vel.data.vx = 0;
			m_vel.data.vy = 0;
			m_vel.data.va = -m_turn_vel;
			m_velOut.write();
			std::cout << " left turn" << std::endl;
			Sleep(t_t * 1000);
		}
		else if (toward == 2)
		{
			m_vel.data.vx = 0;
			m_vel.data.vy = 0;
			m_vel.data.va = m_turn_vel;
			m_velOut.write();
			std::cout << " right turn" << std::endl;
			Sleep(t_t * 1000);
		}
		else{
			m_vel.data.vx = 0;
			m_vel.data.vy = 0;
			m_vel.data.va = 2 * m_turn_vel;
			m_velOut.write();
			std::cout << " back turn" << std::endl;
			Sleep(t_t * 1000);
		}

		m_vel.data.vx = m_front_vel;
		m_vel.data.vy = 0;
		m_vel.data.va = 0;
		changetoward = true;
		toward = 0;
		std::cout << " forward" << std::endl;
	}
	m_velOut.write();


  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ev3touch::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ev3touch::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ev3touch::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ev3touch::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ev3touch::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ev3touchInit(RTC::Manager* manager)
  {
    coil::Properties profile(ev3touch_spec);
    manager->registerFactory(profile,
                             RTC::Create<ev3touch>,
                             RTC::Delete<ev3touch>);
  }
  
};


